import * as d3 from 'd3';
import { globalSetting } from '../global';
import { NetlistRender } from '.';

export class CellRender {
    /**
     * 
     * @param {d3.Selection} selection 
     * @param {NetlistRender} rootRender 
     */
    constructor(selection, rootRender) {
        this.parentSelection = selection;
        this.rootRender = rootRender;

        /**
         * @type {BasicD3DataItem[]}
         */
        this.data = [];
    }

    /**
     * @description 将 elknode 关于 器件 的数据添加为 d3 数据项目
     * @param {ElkNode} node 
     * @param {HTMLElement} element
     */
    addAsD3DataItem(node, element) {
        this.data.push({
            element,
            x: node.x,
            y: node.y,
            width: node.width,
            height: node.height,
            fill: 'var(--main-dark-color)'
        });
    }

    render() {
        const data = this.data;
        const rootRender = this.rootRender;

        let cellSelections = this.parentSelection.selectAll('svg')
            .data(data)
            .enter()
            .append(data => {
                const element = data.element;
                element.setAttribute('x', data.x);
                element.setAttribute('y', data.y);
                if (globalSetting.renderAnimation) {
                    element.setAttribute('stroke-opacity', 0);
                }
                return element;
            });
        
        if (globalSetting.renderAnimation) {
            cellSelections = cellSelections
                .transition()
                .duration(1000)
                .attr('stroke-opacity', 1)
                .attr('class', 'grab')
                .on('end', function (data) {                    
                    const cellSelection = d3.select(this);

                    registerDragEvent(cellSelection, data, rootRender);
                });
        } else {
            cellSelections = cellSelections
                .attr('class', 'grab')
                .each(function (data) {
                    const cellSelection = d3.select(this);

                    registerDragEvent(cellSelection, data, rootRender);
                });
        }

        this.selections = cellSelections;
        return cellSelections;
    }
}

/**
 * @description 注册关于 器件 的拖动事件
 * 
 * 需要提取最小拓扑子图，然后重新调整各个区域的尺寸
 * @param {d3.Selection} selection
 * @param {any} data
 * @param {NetlistRender} rootRender
 */
export function registerDragEvent(selection, data, rootRender) {
    // 创建拖拽行为
    const drag = d3.drag();

    drag.on('start', async event => dragStart(event, selection, data));
    drag.on('drag', async event => dragged(event, selection, data, rootRender));
    drag.on('end', async event => dragEnd(event, selection, data))
    
    selection.call(drag);
}

/**
 * 
 * @param {d3.D3DragEvent} event 
 * @param {d3.Selection} selection 
 */
function dragStart(event, selection, data) {
    selection.attr('class', 'grabbing');
}

/**
 * 
 * @param {d3.D3DragEvent} event 
 * @param {d3.Selection} selection 
 * @param {NetlistRender} rootRender 
 */
function dragged(event, selection, data, rootRender) {
    // 当拖动结束时，D3 会根据绑定的数据（data.x 和 data.y）重新渲染元素，导致元素回到初始位置。
    // 所以需要 手动更新 data.x 和 data.y
    data.x = event.x;
    data.y = event.y;
    
    selection
        .attr('x', event.x)
        .attr('y', event.y);
    
    // 根据最小拓扑图，提取出关键点，重新计算布局

}

/**
 * 
 * @param {d3.D3DragEvent} event 
 * @param {d3.Selection} selection 
 */
function dragEnd(event, selection, data) {
    selection.attr('class', 'grab');
}