import * as d3 from 'd3';
import ELK from 'elkjs';
import { globalLookup, globalSetting } from '../global';

export class PortRender {
    /**
     * 
     * @param {d3.Selection} selection 
     */
    constructor(selection) {
        this.parentSelection = selection;
        /**
         * @type {BasicD3DataItem[]}
         */
        this.data = [];
    }

    /**
     * @description 将 elknode 关于 port 的数据添加为 d3 数据项目
     * @param {ElkNode} node 
     */
    addAsD3DataItem(node) {
        this.data.push({
            x: node.x,
            y: node.y,
            width: node.width,
            height: node.height,
            fill: 'var(--main-dark-color)',
            text: node.renderName,
            rx: 3,
            ry: 3
        });
    }

    render() {
        const data = this.data;

        let portSelections = this.parentSelection.selectAll('g.port')
            .data(data)
            .enter()
            .append('g')
            .attr('class', 'port')
            .attr("transform", d => `translate(${d.x}, ${d.y})`);
        
        let ports = portSelections.append('rect')
            .attr('width', data => data.width)
            .attr('height', data => data.height)
            .attr('fill', d => d.fill);
        
        let texts = portSelections.append('text')
            .attr('x', data => data.width / 2) // 文本的 x 坐标（居中）
            .attr('y', data => data.height / 2) // 文本的 y 坐标（居中）
            .attr('dominant-baseline', 'middle') // 文本垂直居中
            .attr('text-anchor', 'middle') // 文本水平居中
            .attr('fill', 'white')              // 文本颜色
            .attr('font-size', '0')
            .transition()
            .duration(1000)
            .attr('font-size', '12px')
            .attr('class', 'port-caption')
            .text(data => data.text); // 设置文本内容


        if (globalSetting.renderAnimation) {
            ports.transition()
                .duration(1000)
                .attr('stroke', 'var(--main-color)')
                .attr('stroke-width', 2)
                .attr('rx', d => d.rx)
                .attr('ry', d => d.ry);

            portSelections
                .attr('class', 'grab')
                .each(function (data) {                    
                    const portSelection = d3.select(this);
                    registerDragEvent(portSelection, data);
                });
        } else {
            ports.attr('stroke', 'var(--main-color)')
                .attr('stroke-width', 2)
                .attr('rx', d => d.rx)
                .attr('ry', d => d.ry);
                
            portSelections
                .attr('class', 'grab')
                .each(function (data) {
                    const portSelection = d3.select(this);
                    registerDragEvent(portSelection, data);
                });
        }

        
        this.selections = portSelections;
        return portSelections;
    }
}




/**
 * @description 注册关于 器件 的拖动事件
 * 
 * 需要提取最小拓扑子图，然后重新调整各个区域的尺寸
 * @param {d3.Selection} selection
 * @param {any} data
 */
export function registerDragEvent(selection, data) {
    // 创建拖拽行为
    const drag = d3.drag();

    drag.on("start", event => dragStart(event, selection, data));
    drag.on("drag", event => dragged(event, selection, data));
    drag.on("end", event => dragEnd(event, selection, data));
    
    selection.call(drag);
}

/**
 * 
 * @param {d3.D3DragEvent} event 
 * @param {d3.Selection} selection 
 */
function dragStart(event, selection, data) {
    selection.attr('class', 'grabbing');
}

/**
 * 
 * @param {d3.D3DragEvent} event 
 * @param {d3.Selection} selection 
 */
function dragged(event, selection, data) {    
    data.x = event.x;
    data.y = event.y;
    selection.attr("transform", `translate(${event.x}, ${event.y})`);
}

/**
 * 
 * @param {d3.D3DragEvent} event 
 * @param {d3.Selection} selection 
 */
function dragEnd(event, selection, data) {
    selection.attr('class', 'grab');
}